"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hashSchema = exports.createTSInterfaceFile = exports.createTypescriptInterfaceFiles = exports.createJSONSchema = exports.createTypescriptInterfaceDefinition = exports.createMongooseSchema = exports.now = exports.Complex = exports.ObjectId = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const complex_1 = __importDefault(require("./complex"));
const hash_schema_1 = require("./hash-schema");
const json_schema_output_1 = __importDefault(require("./json-schema-output"));
const karhu_1 = require("./karhu");
const mongoose_output_1 = __importDefault(require("./mongoose-output"));
const now_1 = __importDefault(require("./now"));
const object_id_1 = require("./object-id");
const ts_output_1 = __importDefault(require("./ts-output"));
exports.ObjectId = object_id_1.ObjectId;
exports.Complex = complex_1.default;
exports.now = now_1.default;
const log = karhu_1.karhu('schematar-api');
function createMongooseSchema(schema, contextOrOptions = 'mongoose') {
    var _a;
    const options = typeof contextOrOptions === 'string'
        ? { context: contextOrOptions }
        : { ...contextOrOptions, context: (_a = contextOrOptions.context) !== null && _a !== void 0 ? _a : 'mongoose' };
    return mongoose_output_1.default(schema, options);
}
exports.createMongooseSchema = createMongooseSchema;
function createTypescriptInterfaceDefinition(exportName, schema, context = 'typescript', options = {}) {
    return ts_output_1.default(exportName, schema, context, options);
}
exports.createTypescriptInterfaceDefinition = createTypescriptInterfaceDefinition;
function createJSONSchema(schema, context = 'jsonschema', options = {}) {
    return json_schema_output_1.default(schema, context, options);
}
exports.createJSONSchema = createJSONSchema;
async function createTypescriptInterfaceFiles(sourceFileGlobOrFileArray, outputPath, logCreations = false) {
    const { default: glob } = await Promise.resolve().then(() => __importStar(require('glob')));
    await mkdirp_1.default(outputPath);
    const files = [].concat(...(Array.isArray(sourceFileGlobOrFileArray)
        ? sourceFileGlobOrFileArray.map((f) => glob.sync(f))
        : [glob.sync(sourceFileGlobOrFileArray)]));
    for (const file of files) {
        await createTSInterfaceFile(file, outputPath, logCreations);
    }
}
exports.createTypescriptInterfaceFiles = createTypescriptInterfaceFiles;
async function createTSInterfaceFile(filename, outputPath, logCreations = true, formatOutputFilename = defaultOutputFilenameFormatter) {
    const schemaFile = require(path_1.default.resolve(filename));
    const topLevelSchema = schemaFile.schema || schemaFile.default;
    const topLevelName = schemaFile.name || pickNameFromFilename(filename);
    const exported = [];
    if (schemaFile.typescriptSchemas) {
        const typescriptSchemas = schemaFile.typescriptSchemas;
        for (const schemaDef of typescriptSchemas) {
            const schema = schemaDef.schema || topLevelSchema, name = schemaDef.name || topLevelName;
            if (!schema)
                throw new Error('No schema found for ' + name);
            exported.push(createTypescriptInterfaceDefinition(name, schema, schemaDef.context || 'typescript', {
                ...schemaDef,
                doNotImportObjectId: schemaDef.doNotImportObjectId || typescriptSchemas.indexOf(schemaDef) > 0,
            }));
        }
    }
    else if (topLevelSchema) {
        exported.push(createTypescriptInterfaceDefinition(topLevelName, topLevelSchema));
    }
    const outputFilename = path_1.default.join(outputPath, formatOutputFilename(filename));
    if (logCreations) {
        log.info('Storing typescript interfaces in', outputFilename);
    }
    if (exported.length) {
        await new Promise((resolve, reject) => fs_1.default.writeFile(outputFilename, exported.join('\n'), 'utf8', (err) => (!err ? resolve() : reject(err))));
    }
}
exports.createTSInterfaceFile = createTSInterfaceFile;
function defaultOutputFilenameFormatter(inputFilename) {
    const ext = path_1.default.extname(inputFilename), basename = path_1.default.basename(inputFilename, ext);
    return basename + '.schematar.ts';
}
function pickNameFromFilename(filename) {
    const fn = path_1.default.basename(filename, path_1.default.extname(filename));
    return (fn[0].toUpperCase() + fn.substring(1))
        .replace(/[\-_][a-z]/g, (match) => match[1].toUpperCase())
        .replace(/[^a-zA-Z0-9]/g, '');
}
function hashSchema(schema) {
    return hash_schema_1.generateHash(schema);
}
exports.hashSchema = hashSchema;
