"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateHash = void 0;
const crypto_1 = __importDefault(require("crypto"));
const complex_1 = __importDefault(require("./complex"));
const object_id_1 = require("./object-id");
const schema_1 = require("./schema");
function generateHash(schema) {
    if (complex_1.default.isComplex(schema)) {
        return generateHash({ fields: schema.subschema });
    }
    const string = [...fieldsToStrings(schema.fields)].join('\n');
    return hashString(string);
}
exports.generateHash = generateHash;
function* fieldsToStrings(fields) {
    for (const [name, field] of Object.entries(fields)) {
        yield name;
        if (schema_1.isFullDeclaration(field)) {
            yield 'F';
            yield* getTypeString(field.type);
            yield field.enum ? enumToString(field.enum) : '';
            yield field.allowNull ? 'N' : 'R';
            yield (field.optionalIn || []).join(',');
            yield field.optional ? 'O' : 'R';
            yield (field.presentIn || []).join(',');
        }
        else {
            yield 'T';
            yield* getTypeString(field);
        }
    }
}
function* getTypeString(type) {
    if (type === object_id_1.ObjectId)
        return yield 'ObjectId';
    if (type === String)
        return yield 'string';
    if (type === Date)
        return yield 'DateType';
    if (type === Boolean)
        return yield 'boolean';
    if (type === Number)
        return yield 'number';
    if (type === Object)
        return yield 'any';
    if (complex_1.default.isComplex(type)) {
        yield 'Complex{';
        yield* fieldsToStrings(type.subschema);
        yield '}';
    }
    else if (type instanceof Array) {
        yield 'Array[';
        yield* fieldsToStrings({ item: type[0] });
        yield ']';
    }
    if (schema_1.isSchema(type)) {
        yield* getTypeString(new complex_1.default(type.fields));
    }
}
function enumToString(enumValues) {
    return enumValues.map((e) => e.toString()).join(',');
}
function hashString(str) {
    return crypto_1.default.createHash('sha256').update(str).digest('hex');
}
