"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filePrefix = void 0;
const complex_1 = __importDefault(require("./complex"));
const now_1 = __importDefault(require("./now"));
const object_id_1 = require("./object-id");
exports.filePrefix = `import mongoose from 'mongoose'\n// @ts-ignore -- ObjectId may or may not be used \nconst ObjectId = mongoose.Schema.Types.ObjectId\n\n`;
function default_1(exportName, schema, context = 'mongoose') {
    const output = [];
    output.push(`export const ${exportName} = {`);
    for (const field of outputFields(schema.fields, context, '  '))
        output.push(field);
    output.push('}');
    return output.join('\n');
}
exports.default = default_1;
function* outputFields(fields, context, indentation) {
    for (const key of Object.keys(fields)) {
        if (key === '_id' || key === '__v')
            continue;
        const field = fields[key];
        const presentIn = field.presentIn;
        if (presentIn && !presentIn.includes(context))
            continue;
        const ftm = [...outputFieldFormat(field, context, indentation)];
        if (ftm.length === 1) {
            yield `${indentation}${key}: ${ftm[0]},`;
        }
        else {
            yield `${indentation}${key}: ${ftm[0]}`;
            yield* yieldMany(ftm.slice(1, -1));
            yield ftm.slice(-1)[0] + ',';
        }
    }
}
function* outputFieldFormat(field, context, indentation) {
    if (isFullDeclaration(field)) {
        yield '{';
        const subind = indentation + '  ';
        yield `${subind}type: ${asMongooseType(field.type, context, subind)},`;
        if (field.mongoose) {
            throw new Error('Mongoose object is not supported for mongoose file output at this time');
        }
        if (field.index === true) {
            yield `${subind}index: true,`;
        }
        else if (field.index === 'unique') {
            yield `${subind}unique: true,`;
        }
        else if (field.index === 'unique-sparse') {
            yield `${subind}unique: true,`;
            yield `${subind}sparse: true,`;
        }
        if (field.enum) {
            yield `${subind}enum: [${field.enum.map((val) => "'" + val.replace(/'/g, '\\') + "'").join(', ')}],`;
        }
        if (field.mongooseDefault !== undefined) {
            if (field.mongooseDefault === now_1.default) {
                yield `${subind}default: Date.now,`;
            }
            else if (typeof field.mongooseDefault === 'string') {
                yield `${subind}default: '${field.mongooseDefault.replace(/'/, "\\'")}',`;
            }
            else if (typeof field.mongooseDefault === 'number') {
                yield `${subind}default: ${field.mongooseDefault},`;
            }
            else {
                throw new Error('Cannot handle default value: ' + field.mongooseDefault);
            }
        }
        if (field.mongooseRef) {
            yield `${subind}ref: '${field.mongooseRef}',`;
        }
        if (field.mongooseExpires) {
            yield `${subind}expires: '${field.mongooseExpires}',`;
        }
        yield indentation + '}';
    }
    else {
        yield asMongooseType(field, context, indentation);
    }
}
function asMongooseType(type, context, indentation) {
    if (type === object_id_1.ObjectId)
        return 'ObjectId';
    if (type === String)
        return 'String';
    if (type === Boolean)
        return 'Boolean';
    if (type === Number)
        return 'Number';
    if (type === Object)
        return 'Object';
    if (type === Date)
        return 'Date';
    if (complex_1.default.isComplex(type)) {
        return '{\n' + [...outputFields(type.subschema, context, indentation + '  ')].join('\n') + '\n' + indentation + '}';
    }
    if (type instanceof Array) {
        // TODO: support complex types
        const output = [...outputFieldFormat(type[0], context, indentation)].join('\n');
        return '[' + output + ']';
    }
    throw new Error('Unsupported type for mongoose schema ' + type);
}
function isFullDeclaration(field) {
    return !!field.type;
}
function* yieldMany(items) {
    for (const item of items)
        yield item;
}
