"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.outputFields = void 0;
const complex_1 = __importDefault(require("./complex"));
const hash_schema_1 = require("./hash-schema");
const object_id_1 = require("./object-id");
const schema_1 = require("./schema");
function default_1(exportName, schema, context = 'typescript', options = {}) {
    const output = [];
    output.push('// Generated file, do not edit!');
    output.push('');
    output.push('// tslint:disable');
    output.push('/* eslint-disable */');
    output.push('// @ts-ignore -- ignore possibly unused type parameters');
    output.push(`export interface ${exportName}Base<IDType, DateType>`);
    if (complex_1.default.isComplex(schema)) {
        output.push(schema.outputTypescript(context, '  ', null));
    }
    else {
        output.push('{');
        for (const field of outputFields(schema.fields, context, '  ')) {
            output.push(field);
        }
        output.push('}');
    }
    if (!options.omitExtraExports) {
        if (!options.doNotImportObjectId) {
            output.push(`import {ObjectId} from 'mongodb'`);
        }
        output.push(`export type ${exportName}Mongoose = ${exportName}Base<ObjectId, Date>`);
        output.push(`export type ${exportName}JSON = ${exportName}Base<string, string>`);
        output.push(`export type ${exportName}Fluid = ${exportName}Base<string | ObjectId, string | Date>`);
    }
    const { exportHash } = options;
    if (exportHash) {
        const hash = hash_schema_1.generateHash(complex_1.default.isComplex(schema) ? { fields: schema.subschema } : schema);
        output.push(`export const ${exportHash} = '${hash}'`);
    }
    return output.join('\n');
}
exports.default = default_1;
function* outputFields(fields, context, indentation) {
    for (const key of Object.keys(fields)) {
        const field = fields[key];
        const presentIn = field.presentIn;
        if (presentIn && !presentIn.includes(context))
            continue;
        const ftm = [...outputFieldFormat(field, context, indentation)];
        const optional = isOptional(field, context);
        const allowNull = field.allowNull;
        if (allowNull && !optional)
            throw new Error('allowNull can only be used on optional fields');
        if (ftm.length === 1) {
            yield `${indentation}${key}${optional ? '?' : ''}: ${ftm[0]}${allowNull ? ' | null' : ''}`;
        }
        else {
            yield `${indentation}${key}: ${ftm[0]}`;
            yield* yieldMany(ftm.slice(1));
        }
    }
}
exports.outputFields = outputFields;
function isOptional(field, context) {
    if (!field.type)
        return false;
    const fi = field;
    return fi.optional || (fi.optionalIn && fi.optionalIn.includes(context));
}
function* outputFieldFormat(field, context, indentation) {
    if (schema_1.isFullDeclaration(field)) {
        if (field.enum) {
            const isArray = Array.isArray(field.type);
            yield (isArray ? 'Array<' : '') +
                field.enum.map((f) => "'" + f.replace(/'/g, "\\'") + "'").join(' | ') +
                (isArray ? '>' : '');
        }
        else {
            yield asTSType(field, context, indentation);
        }
    }
    else {
        yield asTSType({ type: field }, context, indentation);
    }
}
function asTSType(field, context, indentation) {
    const { type } = field;
    if (type === object_id_1.ObjectId)
        return 'IDType';
    if (type === String)
        return 'string';
    if (type === Date)
        return 'DateType';
    if (type === Boolean)
        return 'boolean';
    if (type === Number)
        return 'number';
    if (type === Object)
        return 'any';
    if (complex_1.default.isComplex(type)) {
        return type.outputTypescript(context, indentation, field);
    }
    if (type instanceof Array) {
        // TODO: support complex types
        const output = [...outputFieldFormat(type[0], context, indentation)].join('\n');
        return 'Array<' + output + '>';
    }
    if (schema_1.isSchema(type)) {
        const asComplex = new complex_1.default(type.fields);
        return asComplex.outputTypescript(context, indentation, { ...field, type: asComplex });
    }
    throw new Error('Unsupported type for typescript type ' + type);
}
function* yieldMany(items) {
    for (const item of items)
        yield item;
}
