import { Schema } from '../../schema'
import { testJSONSchema, testMongooseField, testTypescriptInterface } from '../test-utils'

describe('string-test', function () {
  describe('basic type', function () {
    const schema: Schema = {
      fields: {
        field: String,
      },
    }

    it(
      'json-schema',
      testJSONSchema(schema, (data) => {
        expect(data).toMatchSnapshot()
      })
    )

    it(
      'mongoose',
      testMongooseField(schema, 'field', (field) => {
        expect(field).toEqual({ type: String })
      })
    )

    it(
      'typescript',
      testTypescriptInterface(schema, (tsInterface) => {
        expect(tsInterface).toMatchSnapshot()
      })
    )
  })

  describe('enums are supported', function () {
    const schema: Schema = {
      fields: {
        field: { type: String, enum: ['Alpha', 'Beta', 'Gamma'] },
      },
    }

    it(
      'json-schema',
      testJSONSchema(schema, (data) => {
        expect(data).toMatchSnapshot()
      })
    )

    it(
      'mongoose',
      testMongooseField(schema, 'field', (field) => {
        expect(field).toEqual({ type: String, enum: ['Alpha', 'Beta', 'Gamma'] })
      })
    )

    it(
      'typescript',
      testTypescriptInterface(schema, (tsInterface) => {
        expect(tsInterface).toMatchSnapshot()
      })
    )

    describe('enums are also supported for arrays', function () {
      const arraySchema: Schema = {
        fields: {
          field: { type: [String], enum: ['Alpha', 'Beta', 'Gamma'] },
        },
      }

      it(
        'json-schema',
        testJSONSchema(arraySchema, (data) => {
          expect(data).toMatchSnapshot()
        })
      )

      it(
        'mongoose',
        testMongooseField(arraySchema, 'field', (field) => {
          expect(field).toEqual({ type: [{ type: String }], enum: ['Alpha', 'Beta', 'Gamma'] })
        })
      )

      it(
        'typescript',
        testTypescriptInterface(arraySchema, (tsInterface) => {
          expect(tsInterface).toMatchSnapshot()
        })
      )
    })
  })
})
